@@ -26,13 +26,13 @@ testBadFilenameArg <- function()
     for (f in tmpfiles) checkTrue(!file.exists(f))
 }
 
-testBackup <- function()
+test_backup_to_file <- function()
 {
     backup <- tempfile()
     checkTrue(!file.exists(backup))
     df <- data.frame(foo=1:5, bar=letters[1:5], stringsAsFactors = FALSE)
     checkTrue(dbWriteTable(DATA$db, "t1", df, row.names = FALSE))
-    checkTrue(sqliteCopyDatabase(DATA$db, backup))
+    sqliteCopyDatabase(DATA$db, backup)
     checkTrue(file.exists(backup))
     db2 <- dbConnect(SQLite(), dbname = backup)
     on.exit(dbDisconnect(db2))
@@ -40,3 +40,16 @@ testBackup <- function()
     backupDf <- dbReadTable(db2, "t1")
     checkEquals(df, backupDf)
 }
+
+test_backup_to_connection <- function()
+{
+    df <- data.frame(foo=1:5, bar=letters[1:5], stringsAsFactors = FALSE)
+    checkTrue(dbWriteTable(DATA$db, "t1", df, row.names = FALSE))
+
+    db2 <- dbConnect(SQLite(), dbname = ":memory:")
+    on.exit(dbDisconnect(db2))
+
+    sqliteCopyDatabase(DATA$db, db2)
+    got <- dbReadTable(db2, "t1")
+    checkEquals(df, got)
+}
