@@ -27,6 +27,8 @@ char *compiledVarsion = SQLITE_VERSION;
 int RS_sqlite_import(sqlite3 *db, const char *zTable,
                      const char *zFile, const char *separator, const char *eol, int skip);
 
+void RSQLite_closeResultSet0(RS_DBI_resultSet *result, RS_DBI_connection *con);
+
 /* The macro NA_STRING is a CHRSXP in R but a char * in Splus */
 #define RS_NA_STRING CHAR(NA_STRING)
 
@@ -61,7 +63,7 @@ RS_SQLite_init(SEXP config_params, SEXP reload, SEXP cache)
      */
     RS_DBI_manager *mgr;
     Mgr_Handle mgrHandle;
-    Sint  fetch_default_rec, force_reload, max_con;
+    Sint  fetch_default_rec, force_reload;
     Sint  *shared_cache;
     const char *drvName = "SQLite";
     const char *clientVersion = sqlite3_libversion();
@@ -79,11 +81,18 @@ RS_SQLite_init(SEXP config_params, SEXP reload, SEXP cache)
             "initialization error: must specify max num of conenctions and default number of rows per fetch",
             RS_DBI_ERROR);
     }
-    max_con = INT_EL(config_params, 0);
+    /* max_con is now IGNORED as connections are not tracked by the
+     * manager; instead they are held in external pointers with
+     * finalizers. */
+    /* max_con = INT_EL(config_params, 0); */
     fetch_default_rec = INT_EL(config_params,1);
     force_reload = LGL_EL(reload,0);
 
-    mgrHandle = RS_DBI_allocManager(drvName, max_con, fetch_default_rec,
+    /* The manager does not keep track of connections, so there is no
+       pre-set max connections.  We set this to 1 for now until more
+       of the code is refactored.
+     */
+    mgrHandle = RS_DBI_allocManager(drvName, 1, fetch_default_rec,
                                     force_reload);
 
     mgr = RS_DBI_getManager(mgrHandle);
@@ -227,7 +236,7 @@ RS_SQLite_freeException(RS_DBI_connection *con)
     return;
 }
 
-Con_Handle
+SEXP
 RS_SQLite_newConnection(Mgr_Handle mgrHandle, SEXP dbfile, SEXP allow_ext,
                         SEXP s_flags, SEXP s_vfs)
 {
@@ -235,10 +244,8 @@ RS_SQLite_newConnection(Mgr_Handle mgrHandle, SEXP dbfile, SEXP allow_ext,
     RS_SQLite_conParams *conParams;
     Con_Handle conHandle;
     sqlite3     *db_connection;
-    const char  *dbname = NULL;
-    int         rc, loadable_extensions;
-    const char *vfs = NULL;
-    int open_flags = 0;
+    const char  *dbname = NULL, *vfs = NULL;
+    int         rc, loadable_extensions, open_flags = 0;
 
     if(!is_validHandle(mgrHandle, MGR_HANDLE_TYPE))
         RS_DBI_errorMessage("invalid SQLiteManager", RS_DBI_ERROR);
@@ -306,55 +313,52 @@ RS_SQLite_closeConnection(Con_Handle conHandle)
 {
     RS_DBI_connection *con;
     sqlite3 *db_connection;
-    SEXP status;
     int      rc;
 
     con = RS_DBI_getConnection(conHandle);
     if(con->num_res>0){
-        RS_DBI_errorMessage(
-            "close the pending result sets before closing this connection",
-            RS_DBI_ERROR);
+        /* we used to error out here telling the user to
+           close pending result sets.  Now we warn and close the set ourself.
+         */
+        RS_DBI_errorMessage("closing pending result sets before closing "
+                            "this connection", RS_DBI_WARNING);
+        RSQLite_closeResultSet0(con->resultSets[0], con);
     }
 
     db_connection = (sqlite3 *) con->drvConnection;
     rc = sqlite3_close(db_connection);  /* it also frees db_connection */
-    if(rc==SQLITE_BUSY){
+    if (rc == SQLITE_BUSY) {
+        /* This will happen if there is an unfinalized prepared statement or
+           an unfinalized BLOB reference.  Should not happen under normal
+           operation -- even if user is doing things out of order.
+         */
         RS_DBI_errorMessage(
-            "finalize the pending prepared statements before closing this connection",
-            RS_DBI_ERROR);
+            "unfinalized prepared statements before closing this connection",
+            RS_DBI_WARNING);
     }
     else if(rc!=SQLITE_OK){
-        RS_DBI_errorMessage(
-            "internal error: SQLite could not close the connection",
-            RS_DBI_ERROR);
+        RS_DBI_errorMessage("internal error: "
+                            "SQLite could not close the connection",
+                            RS_DBI_WARNING);
     }
 
     /* make sure we first free the conParams and SQLite connection from
      * the RS-RBI connection object.
      */
-
     if(con->conParams){
         RS_SQLite_freeConParams(con->conParams);
         /* we must set con->conParms to NULL (not just free it) to signal
          * RS_DBI_freeConnection that it is okay to free the connection itself.
          */
-        con->conParams = (RS_SQLite_conParams *) NULL;
+        con->conParams = NULL;
     }
-    /* free(db_connection); */    /* freed by sqlite3_close? */
-    con->drvConnection = (void *) NULL;
-
+    con->drvConnection = NULL;
     RS_SQLite_freeException(con);
-    con->drvData = (void *) NULL;
+    con->drvData = NULL;
     RS_DBI_freeConnection(conHandle);
-
-    PROTECT(status = NEW_LOGICAL((Sint) 1));
-    LGL_EL(status, 0) = TRUE;
-    UNPROTECT(1);
-
-    return status;
+    return ScalarLogical(1);
 }
 
-
 int SQLite_decltype_to_type(const char* decltype)
 {
     unsigned int h = 0;
@@ -415,7 +419,7 @@ int RS_SQLite_get_row_count(sqlite3* db, const char* tname) {
         error("SQL error: %s\n", sqlite3_errmsg(db));
     }
     rc = sqlite3_step(stmt);
-    if (rc != SQLITE_OK) {
+    if (rc != SQLITE_ROW && rc != SQLITE_DONE) {
         sqlite3_finalize(stmt);
         error("SQL error: %s\n", sqlite3_errmsg(db));
     }
@@ -427,20 +431,15 @@ int RS_SQLite_get_row_count(sqlite3* db, const char* tname) {
 
 SEXP RS_SQLite_quick_column(Con_Handle conHandle, SEXP table, SEXP column)
 {
-    SEXP ans = R_NilValue;
+    SEXP ans = R_NilValue, rawv;
     RS_DBI_connection *con = NULL;
-    sqlite3           *db_connection = NULL;
-    int               numrows;
-    char              sqlQuery[500];
-    const char        *table_name = NULL;
-    const char        *column_name = NULL;
-    int               rc;
-    sqlite3_stmt      *stmt = NULL;
-    const char        *tail = NULL;
-    int               i = 0;
-    int               col_type;
-    int              *intans = NULL;
-    double           *doubleans = NULL;
+    sqlite3 *db_connection = NULL;
+    sqlite3_stmt *stmt = NULL;
+    int numrows, rc, i = 0, col_type, *intans = NULL, blob_len;
+    char sqlQuery[500];
+    const char *table_name = NULL, *column_name = NULL, *tail = NULL;
+    double *doubleans = NULL;
+    const Rbyte *blob_data;
 
     con = RS_DBI_getConnection(conHandle);
     db_connection = (sqlite3 *) con->drvConnection;
@@ -460,7 +459,7 @@ SEXP RS_SQLite_quick_column(Con_Handle conHandle, SEXP table, SEXP column)
     }
 
     rc = sqlite3_step(stmt);
-    if (rc != SQLITE_OK) {
+    if (rc != SQLITE_ROW) {
         error("SQL error: %s\n", sqlite3_errmsg(db_connection));
     }
     col_type = sqlite3_column_type(stmt, 0);
@@ -480,7 +479,7 @@ SEXP RS_SQLite_quick_column(Con_Handle conHandle, SEXP table, SEXP column)
         error("RS_SQLite_quick_column: encountered NULL column");
         break;
     case SQLITE_BLOB:
-        error("RS_SQLite_quick_column: BLOB column handling not implementing");
+        PROTECT(ans = allocVector(VECSXP, numrows));
         break;
     default:
         error("RS_SQLite_quick_column: unknown column type %d", col_type);
@@ -498,6 +497,15 @@ SEXP RS_SQLite_quick_column(Con_Handle conHandle, SEXP table, SEXP column)
         case SQLITE_TEXT:
             SET_STRING_ELT(ans, i, /* cast for -Wall */
                            mkChar((char*)sqlite3_column_text(stmt, 0)));
+            break;
+        case SQLITE_BLOB:
+            blob_data = (const Rbyte *) sqlite3_column_blob(stmt, 0);
+            blob_len = sqlite3_column_bytes(stmt, 0);
+            PROTECT(rawv = allocVector(RAWSXP, blob_len));
+            memcpy(RAW(rawv), blob_data, blob_len * sizeof(Rbyte));;
+            SET_VECTOR_ELT(ans, i, rawv);
+            UNPROTECT(1);
+            break;
         }
         i++;
         rc = sqlite3_step(stmt);
@@ -507,10 +515,8 @@ SEXP RS_SQLite_quick_column(Con_Handle conHandle, SEXP table, SEXP column)
     return ans;
 }
 
-
-static void RSQLite_freeResultSet(Res_Handle rsHandle)
+static void RSQLite_freeResultSet0(RS_DBI_resultSet *result, RS_DBI_connection *con)
 {
-    RS_DBI_resultSet  *result = RS_DBI_getResultSet(rsHandle);
     if (result->drvResultSet) {
         sqlite3_finalize((sqlite3_stmt *)result->drvResultSet);
         result->drvResultSet = NULL;
@@ -521,7 +527,7 @@ static void RSQLite_freeResultSet(Res_Handle rsHandle)
         RS_SQLite_freeParameterBinding(params);
         result->drvData = NULL;
     }
-    RS_DBI_freeResultSet(rsHandle);
+    RS_DBI_freeResultSet0(result, con);
 }
 
 /* Helper function to clean up and report an error during a call to
@@ -540,7 +546,7 @@ exec_error(const char *msg,
     snprintf(buf, sizeof(buf), "%s: %s", msg, db_msg);
     RS_SQLite_setException(con, errcode, buf);
     if (rsHandle) {
-        RSQLite_freeResultSet(rsHandle);
+        RSQLite_freeResultSet0(RS_DBI_getResultSet(rsHandle), con);
         rsHandle = NULL;
     }
     if (params) {
@@ -576,11 +582,12 @@ static int
 bind_params_to_stmt(RS_SQLite_bindParams *params,
                     sqlite3_stmt *db_statement, int row)
 {
-    int state, j;
+    int state = SQLITE_OK, j;
     for (j = 0; j < params->count; j++) {
-        SEXP pdata = VECTOR_ELT(params->data, j);
+        SEXP pdata = VECTOR_ELT(params->data, j), rawv;
         int integer;
         double number;
+        Rbyte *raw;
         const char *string;
 
         switch(TYPEOF(pdata)){
@@ -598,6 +605,16 @@ bind_params_to_stmt(RS_SQLite_bindParams *params,
             else
                 state = sqlite3_bind_double(db_statement, j+1, number);
             break;
+        case VECSXP:            /* BLOB */
+            rawv = VECTOR_ELT(pdata, row);
+            if (rawv == R_NilValue) {
+                state = sqlite3_bind_null(db_statement, j+1);
+            } else {
+                raw = RAW(rawv);
+                state = sqlite3_bind_blob(db_statement, j+1,
+                                          raw, LENGTH(rawv), SQLITE_STATIC);
+            }
+            break;
         case STRSXP:
             /* falls through */
         default:
@@ -683,7 +700,7 @@ Res_Handle RS_SQLite_exec(Con_Handle conHandle, SEXP statement, SEXP bind_data)
     if (con->num_res>0) {
         Sint res_id = (Sint) con->resultSetIds[0]; /* SQLite has only 1 res */
         rsHandle = RS_DBI_asResHandle(MGR_ID(conHandle),
-                                      CON_ID(conHandle), res_id);
+                                      CON_ID(conHandle), res_id, conHandle);
         res = RS_DBI_getResultSet(rsHandle);
         if (res->completed != 1) {
             free(dyn_statement);
@@ -808,7 +825,10 @@ RS_SQLite_createDataMappings(Res_Handle rsHandle)
             error("NULL column handling not implemented");
             break;
         case SQLITE_BLOB:
-            error("BLOB column handling not implemented");
+            flds->type[j] = SQLns_TYPE_BLOB;
+            flds->Sclass[j] = VECSXP;
+            flds->length[j] = (Sint) -1;   /* unknown */
+            flds->isVarLength[j] = (Sint) 1;
             break;
         default:
             error("unknown column type %d", col_type);
@@ -831,8 +851,9 @@ RS_SQLite_createDataMappings(Res_Handle rsHandle)
 static void fill_one_row(sqlite3_stmt *db_statement, SEXP output, int row_idx,
                          RS_DBI_fields *flds)
 {
-    int j;
-    int null_item;
+    int j, null_item, blob_len;
+    SEXP rawv;
+    const Rbyte *blob_data;
 
     for (j = 0; j < flds->num_fields; j++) {
         null_item = (sqlite3_column_type(db_statement, j) == SQLITE_NULL);
@@ -851,6 +872,18 @@ static void fill_one_row(sqlite3_stmt *db_statement, SEXP output, int row_idx,
                 LST_NUM_EL(output,j,row_idx) =
                     sqlite3_column_double(db_statement, j);
             break;
+        case VECSXP:            /* BLOB */
+            if (null_item) {
+                rawv = R_NilValue;
+            } else {
+                blob_data = (const Rbyte *)sqlite3_column_blob(db_statement, j);
+                blob_len = sqlite3_column_bytes(db_statement, j);
+                PROTECT(rawv = allocVector(RAWSXP, blob_len));
+                memcpy(RAW(rawv), blob_data, blob_len * sizeof(Rbyte));
+            }
+            SET_VECTOR_ELT(VECTOR_ELT(output, j), row_idx, rawv);
+            if (rawv != R_NilValue) UNPROTECT(1);
+            break;
         case STRSXP:
             /* falls through */
         default:
@@ -935,6 +968,7 @@ SEXP RS_SQLite_fetch(SEXP rsHandle, SEXP max_rec)
 
     /* We need to step once to be able to create the data mappings */
     state = do_select_step(res, row_idx);
+    db_statement = (sqlite3_stmt *)res->drvResultSet;
     if (state != SQLITE_ROW && state != SQLITE_DONE) {
         char errMsg[2048];
         (void)sprintf(errMsg, "RS_SQLite_fetch: failed first step: %s",
@@ -957,8 +991,6 @@ SEXP RS_SQLite_fetch(SEXP rsHandle, SEXP max_rec)
 
     PROTECT(output = NEW_LIST((Sint) num_fields));
     RS_DBI_allocOutput(output, flds, num_rec, 0);
-
-    db_statement = (sqlite3_stmt *)res->drvResultSet;
     while (state != SQLITE_DONE) {
         fill_one_row(db_statement, output, row_idx, flds);
         row_idx++;
@@ -1206,14 +1238,26 @@ RS_SQLite_getException(SEXP conHandle)
     return output;
 }
 
+void RSQLite_closeResultSet0(RS_DBI_resultSet *result, RS_DBI_connection *con)
+{
+   if(result->drvResultSet == NULL)
+       RS_DBI_errorMessage("corrupt SQLite resultSet, missing statement handle",
+                           RS_DBI_ERROR);
+    RSQLite_freeResultSet0(result, con);
+}
+
 SEXP 
 RS_SQLite_closeResultSet(SEXP resHandle)
 {
-    RS_DBI_resultSet *result = RS_DBI_getResultSet(resHandle);
-    if(result->drvResultSet == NULL)
-        RS_DBI_errorMessage("corrupt SQLite resultSet, missing statement handle",
-                            RS_DBI_ERROR);
-    RSQLite_freeResultSet(resHandle);
+    RSQLite_closeResultSet0(RS_DBI_getResultSet(resHandle),
+                            RS_DBI_getConnection(resHandle));
+    /* The connection external ptr is stored within the result handle
+       so that an active result keeps the connection protected.  When
+       we close the result set, we remove the reference to the
+       connection so that the connection can be gc'd.
+     */
+    SET_VECTOR_ELT(R_ExternalPtrProtected(resHandle), 1, R_NilValue);
+    RES_ID(resHandle) = -1;
     return ScalarLogical(1);
 }
 
@@ -1254,7 +1298,8 @@ RS_SQLite_managerInfo(Mgr_Handle mgrHandle)
 
     cons = (Sint *) S_alloc((long)max_con, (int)sizeof(Sint));
     ncon = RS_DBI_listEntries(mgr->connectionIds, mgr->length, cons);
-    if(ncon != num_con){
+    /* expecting ncon == 0 alwasy */
+    if(ncon != 0){
         RS_DBI_errorMessage(
             "internal error: corrupt RS_DBI connection table",
             RS_DBI_ERROR);
@@ -1644,4 +1689,3 @@ SEXP RS_SQLite_copy_database(Con_Handle fromConHandle, Con_Handle toConHandle)
     }
     return R_NilValue;
 }
-
